const {isValue} = require('@icode-run/utils/cjs/utils');
const {getOutputValue} = require('@icode-run/utils/cjs/dataType');
const request = require("request");
const fs = require("fs");
const path = require("path");
const qs = require("qs");

/**
 * 把组件原配置中的staticData转化成组件props.data数据
 * @param {*} dataDescription 静态数据（表格数据）
 * @returns 返回转化后的数据，用于组件显示
 */
const _getStaticData = (dataDescription) => {
    const staticData = [];
    dataDescription.forEach((rowData, rowIndex) => {
      if(rowIndex > 0) {
        const rowItem = {};
        (rowData || []).map((colData, colIndex) => {
          rowItem[colIndex + ''] = colData;
        })
        staticData.push(rowItem);
      }
    })
  
    return staticData;
  }
exports.getStaticData = _getStaticData;

const _getDefaultValue = (keyName, defaultValueConfig) => {
  if (!(defaultValueConfig && defaultValueConfig.ruleType && isValue(defaultValueConfig.ruleValue))) {
    return;
  }
  let value = "", type = "";
  if (defaultValueConfig.ruleType === "static") {
    if (
      Number(defaultValueConfig.ruleValue) ==
      defaultValueConfig.ruleValue
    ) {
      type = "number";
    } else {
      type = "string";
    }
    value = defaultValueConfig.ruleValue;
  } else {
    value = `getDefaultValue('${defaultValueConfig.ruleType}', '${defaultValueConfig.ruleValue}')`;
    type = "fn";
  }
  return {
    keyName: keyName,
    value: value,
    type: type,
  }
}
exports.getDefaultValue = _getDefaultValue;

/**
 * 获取初始化组件的默认值
 * ——静态数据(state.DefaultValue = currRelation.defaultValueConfig)
 * ——动态数据, 未给param绑定参数项(state.DefaultValue = currRelation.defaultValueConfig)
 * ——动态数据，已给param绑定参数项(state.Params = currRelation.defaultValueConfig
 * @param {*} propsOption 
 * @param {*} currRelation 
 */
exports.getParamDefaultValueState = ({
  paramStates, 
  propsOptions, 
  currRelation,
  relations,
  dataModels,
}) => {
  paramStates = paramStates || {};
  const staticDefaultValue = currRelation.defaultValueConfig || {};
  
  if(!Object.keys(staticDefaultValue).length) {
    return;
  }
  (propsOptions || []).forEach(options => {
    if(options.type === 'param' && staticDefaultValue[options.name]) {
      if(options.value && options.value.length) {
        options.value.map(valueItem => {
          const paramsName = exports.getApiParamStateName({
            dataModel: dataModels[valueItem.serviceName],
          });
          paramStates[paramsName] = paramStates[paramsName] || {};
          let paramValue = _getDefaultValue(valueItem.keyName, staticDefaultValue[options.name]);
          const newParamValue = getOutputValue({
            outputType: valueItem.structureType,
            value: paramValue.value,
            ...(valueItem.transformType && isValue(valueItem.transformType.index) ? {
              targetIndex: valueItem.transformType.index
            }:{}),
            ...(valueItem.transformType && valueItem.transformType.join ? {
              valueSplit: valueItem.transformType.join
            }:{}),
          })
          paramStates[paramsName][valueItem.keyName] = {
            keyName: valueItem.keyName,
            value: newParamValue,
            type: valueItem.structureType,
          };
        })
        
      }
      const defaultName = `${exports.getApiStateName(currRelation.nodeId, {relations, dataModels})}DefaultValue`;
      paramStates[defaultName] = paramStates[defaultName] || {};
      paramStates[defaultName][options.name] = {
        ..._getDefaultValue(options.name, staticDefaultValue[options.name]),
        type: options.outputType,
      };
      
    }
  })

}

/**
 * 
 */
exports.getNewEvents = ({
  events = [], 
  dataModels,
  relations,
  paramDefaultValueState = {},
}) => {
  const currentEvents = {};
  let stateNodeId, modalNodeId;
  events.map((item) => {
    if (item.eventName && item.eventType && (item.eventContent || item.stateNodeId)) {
      currentEvents[item.eventName] = currentEvents[item.eventName] || [];
      if(item.stateNodeId) {
        stateNodeId = exports.getApiStateName(item.stateNodeId, {dataModels, relations});
      }
      const relationService = [];
      Object.keys(dataModels).forEach(dataModelName => {
        const dataModel = dataModels[dataModelName] || {};
        const apiParams = dataModel.apiParams || {};
        Object.keys(apiParams).forEach(keyName => {
          const defaultValueConfig = (apiParams[keyName] && apiParams[keyName].defaultValueConfig) || {};
          if(defaultValueConfig.ruleType === 'event' && defaultValueConfig.ruleValue && defaultValueConfig.ruleValue[0] === item.stateNodeId) {
            const serviceName = exports.getApiServiceName(dataModel);
            if(serviceName) {
              relationService.push({
                serviceName,
                keyName: defaultValueConfig.ruleValue[1]
              });
            }
          }
        })
      })
      
      let eventContent = item.eventContent;
      if(['openModal', 'closeModal'].indexOf(item.eventType) >= 0) {
        modalNodeId = exports.getNewNodeId(item.eventContent);
        eventContent = `${exports.getApiStateName(item.eventContent, {relations, dataModels})}Visible`;
      }
      
      if('refresh' === item.eventType) {
        const nodeId = item.eventContent;
        const relation = relations[nodeId];
        const dataModel = dataModels[relation.dataSourceName] || {};
        if(relation.dataSourceType === 'api' && relation.dataSourceName && dataModels[relation.dataSourceName]) {
          const paramStateName = exports.getApiParamStateName({
            dataModel,
          });

          const currParamsDefaultValue = paramDefaultValueState[paramStateName] || {};
          const params = {};
          if(item.resetParams && item.resetParams.length) {
            item.resetParams.forEach(keyName => {
              params[keyName] = currParamsDefaultValue[keyName] || currParamsDefaultValue[keyName] === 0 ? currParamsDefaultValue[keyName] : '';
            })
          }
          
          relationService.push({
            serviceName: exports.getApiServiceName(dataModel),
            params: params
          })
        }
      }

      currentEvents[item.eventName].push({
        eventType: item.eventType,
        pageParams: item.pageParams,
        modalNodeId: modalNodeId || '',
        ...(eventContent ? {eventContent} : {}),
        ...(relationService.length ? {
          relationService,
        } : {}),
        ...(item.stateNodeId ? {
          stateNodeId: `${exports.getApiStateName(item.stateNodeId, {relations, dataModels})}EventData`
        } : {}),
        ...(item.messageContent ? {
          messageContent: item.messageContent
        } : {}),
        ...(item.messageDuration ? {
          messageDuration: item.messageDuration
        } : {}),
      });
    }
  });
  return {
    events: currentEvents,
    stateNodeId
  };
}

/**
 * 给dataModels更新dataKeyName的值，优化了构建后的函数名称
 * 返回无重复的dataModel，以便优化构建service.ejs
 */
let dataModelsNames = {};
exports.setNewDataModelName = (dataModels) => {
  let key;
  const newDataModels = [];
  for(key in dataModels) {
    const name = dataModels[key].name;
    if(!dataModelsNames[name]) {
      newDataModels.push(dataModels[key]);
    }
    dataModels[key].dataKeyName = name;
    dataModels[key].dataModelName = key;
    dataModelsNames[name] = (dataModelsNames[name] || 0) + 1;
  }
  return newDataModels;
}

/**
 * 把旧的nodeId换成新的nodeId
 */
let nodeIds = {}, newNodeIdCount = {}, oldNodeIds = {};
exports.setNewNodeId = (item) => {
  const nodeId = item.nodeId;
  const componentName = item.componentName;
  newNodeIdCount[componentName] = (newNodeIdCount[componentName] || 0) + 1;
  nodeIds[nodeId] = `${componentName}${newNodeIdCount[componentName]}`;
  oldNodeIds[nodeIds[nodeId]] = nodeId;
  oldNodeIds[nodeId] = nodeId;
  return nodeIds[nodeId];
};

/**
 *  获取当前relation, 根据新/旧的nodeId，找到relation中旧的nodeId相关的数据;
 */
exports.getCurrentRelations = (nodeId, relations = {}) => {
  // if(relations[oldNodeIds]) {
  //   relations[oldNodeIds].nodeId = nodeId;
  // }
  return relations[oldNodeIds[nodeId] || nodeId] || {};
}

/**
 *  获取Api请求函数名称, 用旧的nodeId换新的nodeId
 */
exports.getApiServiceName = (dataModel = {}) => {
  if(!dataModel.nodeId) {
    return '';
  }
  const newNodeId = nodeIds[dataModel.nodeId] || dataModel.nodeId;
  return `${dataModel.type}${newNodeId}`;
};

/**
 *  获取Api相关数据的state名称, 用旧的nodeId换新的nodeId
 * from: 'update'-dataModel==='update'时；
 */
exports.getApiStateName = (nodeId, {
  dataModels = {}, 
  relations = {},
  from = '',
}) => {
  const newNodeId = nodeIds[nodeId] || nodeId || 'nodeId';
  return from === 'update' ? `update${newNodeId}` : `${newNodeId.charAt(0).toLowerCase() + newNodeId.slice(1)}`;
};

exports.getApiParamStateName = ({
  dataModel
}) => {
  const nodeId = dataModel.nodeId;
  const type = dataModel.type;
  const newNodeId = nodeIds[nodeId] || nodeId || 'nodeId';
  const name = type === 'update' ? `update${newNodeId}` : `${newNodeId.charAt(0).toLowerCase() + newNodeId.slice(1)}`;
  return `${name}Params`;
};

/**
 *  获取Api相关数据的state名称, 用旧的nodeId换新的nodeId
 * from: 'update'-dataModel==='update'时；
 */
 exports.getNewNodeId = (nodeId) => {
  const newNodeId = nodeIds[nodeId] || nodeId;
  return newNodeId ? `${newNodeId.charAt(0).toLowerCase() + newNodeId.slice(1)}` : '';
};

/**
 *  接口请求成功后，更新相关联的“其它接口参数默认值”的当前值
 *  return 需要更新的参数字符串
 */
// exports.getNeedUpdateParams = (currDataModelName, {
//   dataModels = {},
// }) => {
//   let needUpdateParams = '';
//   Object.keys(dataModels).forEach(dataModelsName => {
//     const dataModel = dataModels[dataModelsName] || {};
//     const apiParams = dataModel.apiParams || {};
//     let paramsString = '';
//     Object.keys(apiParams).forEach(keyName => {
//       const defaultValueConfig = (apiParams[keyName] && apiParams[keyName].defaultValueConfig) || {};
//       if(
//         defaultValueConfig.ruleType === 'api' && 
//         defaultValueConfig.ruleValue &&  
//         defaultValueConfig.ruleValue[0] === currDataModelName && 
//         defaultValueConfig.ruleValue[1] 
//       ) {
//         paramsString += `${keyName}: data['${defaultValueConfig.ruleValue[1]}'],\n`;
//       }
//     })
//     if(paramsString) {
//       needUpdateParams += `${exports.getApiParamStateName({dataModel})}: {
//         ${paramsString}
//       },\n`
//     }
//   })
//   return needUpdateParams;
// }

exports.getCss = (cssContent) => {
  const splitNames = cssContent.match(/[a-zA-Z0-Z&-_>: ]*\s*\{/g);
  const indexs = splitNames.map((name) => {
      return cssContent.indexOf(name)
  });
  let currCss = [], otherCss = [];
  indexs.forEach((indexName, index) => {
    const isCurr = splitNames[index].match(/^\&/);
    if(isCurr && isCurr.length && indexs[index + 1]) {
        currCss.push(cssContent.substring(indexName, indexs[index + 1] - 1));
    }else {
        otherCss.push(cssContent.substring(indexName, indexs[index + 1] ? indexs[index + 1] - 1 : cssContent.length));
    }
  })
  return {
    '&': currCss.join('\n'),
    'global': otherCss.join('\n'),
  }
}

exports.getStaticDataName = (nodeId) => {
  return `staticData${nodeId}`;
}

/**
 * 定时更新数据
 * @param {*} refreshData 
 * @returns 
 */
exports.getRefreshDataHtml = (refreshData) => {
  let timeing = 1;
  const apis = {};
  const statics = {};
  refreshData.forEach(item => {
    if(item.refreshData && item.refreshData.timing) {
      timeing = item.refreshData.timing > timeing ? item.refreshData.timing : timeing;
    }
    if(item.type === 'api' && item.serviceName) {
      apis[item.serviceName] = `this.${item.serviceName}()`;
    }
    if(item.type === 'static' && item.stateName && item.refreshData && item.refreshData.config) {
      statics[item.stateName] = `refreshStaticData(this.state.${item.stateName},${JSON.stringify(item.refreshData.config)}),`;
    }
  })
  const staticsParams = Object.keys(statics).map(stateName => {
    return `${stateName}: ${statics[stateName]}`
  });
  if(Object.values(apis).length || staticsParams.length) {
    return `
  refreshTimeValue = null;
  refreshData = () => {
    if(this.refreshTimeValue) {
      clearTimeout(this.refreshTimeValue);
      this.refreshTimeValue = null;
    }
    this.refreshTimeValue = setTimeout(() => {
      clearTimeout(this.refreshTimeValue);
      this.refreshTimeValue = null;
      ${Object.values(apis).join('\n')}
      ${staticsParams.length ? ['this.setState({'].concat(staticsParams, ['});']).join('\n') : ''}

      this.refreshData();
    }, ${timeing * 1000});
  }
  `;
  }
  else {
    return '';
  }
}

exports.getRequestStructuresType = (dataModel={}, keyName) => {
  const requestStructures = dataModel.requestStructures || [];
  let type;
  requestStructures.forEach(item => {
    if(item.keyName === keyName) {
      type = item.type;
    }
  })
  return type;
}


exports.saveImgToLocation = ({
  url,
  imagesDirPath
}) => {
  if(url.match(/^http(|s)\:\/\//) && imagesDirPath) {
    const filenames = url.split('/');
    const filename = `online_${filenames[filenames.length - 1]}`;
    if (!fs.existsSync(imagesDirPath)) {
      fs.mkdirSync(imagesDirPath);
    }
    request.get({uri:url, encoding:'binary'},function(err,res){
      if(!err) {
        fs.writeFile(path.join(imagesDirPath, filename),res.body,"binary",function(err,res){
          
        })
      }
    })
    return `/assets/${filename}`;
  }
  return url;
};

exports.saveGlobalState = ({
  globalState = {},
  globalDirPath
}) => {
  if(globalState.isAppTheme && globalDirPath) {
    const filePath = path.join(globalDirPath, 'globalTheme.json');
    if (!fs.existsSync(globalDirPath)) {
      fs.mkdirSync(globalDirPath);
    }
    fs.writeFileSync(filePath,JSON.stringify(globalState),'utf-8');
    return {
      dependencies: "import globalTheme from '@/layouts/globalTheme.json';",
      content: 'globalTheme'
    }
  }

  return {
    content: JSON.stringify(globalState)
  };
}

exports.replaceImgUrlInCss =  (content, imagesDirPath) => {
  if(!imagesDirPath) {
    return content;
  }
  let newContent = content.replace(/url\(('|")*\/staticImages\/(.(?!url))*\)/g, (value) => {
    let url = value.replace(/^url\(('|")*/, '');
    url = url.replace(/('|")*\)$/, '');
    url = exports.saveImgToLocation({
      url: `https://www.icoderun.com${url}`,
      imagesDirPath,
    })
    return `url(${url})`;
  });
  return newContent;
}

exports.replaceImgUrlInProps = (url = '', imagesDirPath) => {
  if(!imagesDirPath || !url) {
    return url;
  }
  let newUrl;
  let isSaveImg = false;
  if(url.match(/^\/staticImages\//)) {
    isSaveImg = true;
    newUrl = `https://www.icoderun.com${url}`;
  }
  else if(url.match(/^http:\/\/127\.0\.0\.1:\d+\/assets\//)) {
    newUrl = url ? url.replace(/http:\/\/127\.0\.0\.1:\d+\/assets\//, '/assets/') : '';
  }
  else if(url.match(/^http(|s)*\:\/\//)) {
    isSaveImg = true;
    newUrl = url;
  }
  if(isSaveImg) {
    newUrl = exports.saveImgToLocation({
      url: newUrl,
      imagesDirPath: imagesDirPath,
    })
  }
  
  return newUrl;
}

exports.buildComponentDependenciesHtml = ({
  dependencies = {},
  versions = {},
  remoteEntryPaths= {},
  isLoadingNpm = false,
  packagePath,
}) => {
  let html = '';
  const packageDependencies = {};
  let remoteApps = '';
  Object.keys(dependencies).forEach(componentName => {
    const item = dependencies[componentName] || {}
    if(isLoadingNpm) {
      html += `import ${item.componentName} from '${item.packageName}';\n`;
      const version = versions[item.packageName];
      packageDependencies[item.packageName] = `${version ? '^'+version : '*'}`;
    }
    else if(remoteEntryPaths[item.remoteAppName]){
      remoteApps += `${item.remoteAppName}: '${remoteEntryPaths[item.remoteAppName]}',\n`;
      html += `const ${item.componentName} = importRemote('${item.remoteAppName}/${item.componentName}');\n`;
    }
  })

  if(isLoadingNpm && packagePath) {
    const filePath = path.join(packagePath, 'package.json');
    const content = fs.readFileSync(filePath, 'utf-8');
    const data = JSON.parse(content);
    data.dependencies = {
      ...data.dependencies,
      ...packageDependencies,
    };
    fs.writeFileSync(filePath, JSON.stringify(data), 'utf-8');
  }
  else if(remoteApps){
    let remoteEntry = `
      import importRemote, { setRemoteConfig } from 'import-remote-module';
      setRemoteConfig({
        ${remoteApps}
      });
    `;
    
    html = `${remoteEntry}
    ${html}`
  }
  else {
    html = '';
  }
  return html;
}

exports.getCurrApiData = ({
  nodeId,
  relations,
  dataModels,
}) => {
  const currRelation = exports.getCurrentRelations(nodeId, relations);
  if((currRelation.dataSourceType === 'api' || currRelation.dataSourceType === 'state') && currRelation.dataSourceName) {
    return dataModels[currRelation.dataSourceName] && dataModels[currRelation.dataSourceName].apiData;
  }
  else if(currRelation.dataSourceType === 'scope' && currRelation.scopeNodeId) {
    return exports.getCurrApiData({
      nodeId: currRelation.scopeNodeId,
      relations,
      dataModels
    })
  }
  return null;
}

exports.getDefaultValueConfigValue = ({
  getEventValue,
  getApiValue,
  config,
}) => {
  let value = '';
  if(config.ruleType === 'event' && config.ruleValue && config.ruleValue.length === 2 && getEventValue) {
    value = getEventValue(config.ruleValue);;
  }
  else if(config.ruleType === 'api' && config.ruleValue && config.ruleValue.length === 2 && getApiValue) {
    value = getApiValue(config.ruleValue);
  }
  else if(config.ruleType === 'static' && isValue(config.ruleValue)) {
    value = isValue(config.ruleValue) ? config.ruleValue : ''
  }else if(config.ruleType === 'pageParam' && isValue(config.ruleValue)) {
    value = `___this.props.history.location.params && this.props.history.location.params.${config.ruleValue}___`
  }
  return value;
}

exports.pushDefaultValue = ({
  scopeNodeId, // event
  getEventValue,

  serviceName, // api
  getApiValue,

  dataModels = {},
  relations = {},
  requiredRuleType = [],
}) => {
  const updateState = {};
  Object.keys(relations).forEach(key => {
    const relation = relations[key];
    if(relation.defaultValueConfig) {
      Object.keys(relation.defaultValueConfig).forEach(name => {
        const config = relation.defaultValueConfig[name];

        const value = exports.getDefaultValueConfigValue({
          getEventValue,
          getApiValue,
          config
        });

        if(requiredRuleType.indexOf(config.ruleType) >= 0) {
          const newNodeId = exports.getNewNodeId(config.ruleValue[0]);
          // if(
          //   (serviceName === '*' && config.ruleType === 'api') || 
          //   (scopeNodeId === '*' && config.ruleType === 'event')
          // ) {
          //   const stateName = `${exports.getNewNodeId(relation.nodeId)}DefaultValue`;
          //   updateState[stateName] = updateState[stateName] || {};
          //   updateState[stateName][name] = value;
          // }

          // 只设置当前事件所关联的默认值: 
          // 如点击table中的“修改”按钮后，
          // 编利relations，根据relation.defaultValueConfig，把当前行的table数据push到DefaultValue、Param中
          if(
            (scopeNodeId && scopeNodeId === newNodeId && config.ruleType === 'event') || 
            (serviceName && serviceName === config.ruleValue[0] && config.ruleType === 'api') ||
            config.ruleType === 'static' || 
            config.ruleType === 'pageParam'
          ) {
            const paramValues = config.paramValues || [];
            paramValues.forEach(paramValue => {
              const dataModel = dataModels[paramValue.serviceName] || {};
              const apiParamStateName = exports.getApiParamStateName({dataModel});
              updateState[`${apiParamStateName}`] = updateState[`${apiParamStateName}`] || {};
              updateState[`${apiParamStateName}`][paramValue.keyName] = value;
            })
            const stateName = `${exports.getNewNodeId(relation.nodeId)}DefaultValue`;
            updateState[stateName] = updateState[stateName] || {};
            updateState[stateName][name] = value;
          }
        }
        
      })
    }
  })

  // if(serviceName === '*') {
  //   return updateState;
  // }

  Object.keys(dataModels).forEach(key => {
    const dataModel = dataModels[key];
    Object.keys(dataModel.apiParams || {}).forEach(keyName => {
      const currServiceName = exports.getApiParamStateName({dataModel});
      const config = dataModel.apiParams[keyName].defaultValueConfig || {};
      const value = exports.getDefaultValueConfigValue({
        getEventValue,
        getApiValue,
        config
      });

      if(
        requiredRuleType.indexOf(config.ruleType) >= 0 && 
        !(updateState[`${currServiceName}`] && updateState[`${currServiceName}`][keyName])
      ) {
        const newNodeId = exports.getNewNodeId(config.ruleValue[0]);
        if(
          (scopeNodeId && scopeNodeId === newNodeId && config.ruleType === 'event') || 
          (serviceName && serviceName === config.ruleValue[0] && config.ruleType === 'api') || 
          (config.ruleType === 'static' && isValue(config.ruleValue)) || 
          (config.ruleType === 'pageParam' && isValue(config.ruleValue))
        ) {
          updateState[`${currServiceName}`] = updateState[`${currServiceName}`] || {};
          updateState[`${currServiceName}`][keyName] = value;
        }
      }
    })
  })

  return updateState;
}

exports.getFormValue = ({
  componentConfig,
  modalNodeId,
  relations,
  dataModels,
  getEventValue,
  getApiValue,
  isInModal = false,
  isInForm = false,
  formNodeId,
  useDependencies = {},
  resultData = {},
}) => {
  let submitData = {}, valueInSubmitData = {}, defaultValue = {};
  let parentId, isFormChild = false;
  componentConfig.forEach(componentItem => {
    const currNodeId = exports.getNewNodeId(componentItem.nodeId);
    if(!isInModal && modalNodeId === currNodeId) {
      parentId = modalNodeId;
    }
    if(isInModal && !isInForm && componentItem.componentName === 'Form') {
      isFormChild = true;
      formNodeId = currNodeId;
    }
    if(isInModal && isInForm) {
      const currRelation = exports.getCurrentRelations(componentItem.nodeId, relations);

      if(currRelation && currRelation.nodeId) {
        if(componentItem.propsOptions) {
          const defaultValueConfig = currRelation.defaultValueConfig || {};

          componentItem.propsOptions.forEach(propsOption => {
            if(propsOption.type === 'param') {
              let config = defaultValueConfig[propsOption.name] || {};
              const paramValue = propsOption.value && propsOption.value[0] || {};
              const keyName = paramValue.keyName;
              if(!config.ruleType && paramValue.serviceName && keyName) {
                const dataModel = dataModels[paramValue.serviceName];
                config = dataModel.apiParams && dataModel.apiParams[keyName] && dataModel.apiParams[keyName].defaultValueConfig || {};
              }
              const value = exports.getDefaultValueConfigValue({
                getEventValue,
                getApiValue,
                config
              });
              
              if(keyName) {
                resultData[formNodeId] = {
                  ...resultData[formNodeId],
                }
                resultData[formNodeId][keyName] = value;
                valueInSubmitData[`${currNodeId}DefaultValue_${propsOption.name}`] = [`${currNodeId}DefaultValue`, propsOption.name];
              }
              else {
                defaultValue[`${currNodeId}DefaultValue`] = defaultValue[`${currNodeId}DefaultValue`] || {};
                defaultValue[`${currNodeId}DefaultValue`][propsOption.name] = value;
              }
              
            }
          })
        }
      }
      
    }
    if(componentItem.children) {
      const childUpdateRelations = exports.getFormValue({
        componentConfig: componentItem.children,
        modalNodeId,
        relations,
        dataModels,
        getEventValue,
        getApiValue,
        isInModal: parentId || isInModal ? true : false,
        isInForm: isFormChild || isInForm,
        formNodeId,
        useDependencies,
        resultData,
      });

      valueInSubmitData = {
        ...childUpdateRelations.valueInSubmitData,
        ...valueInSubmitData,
      }

      defaultValue = {
        ...childUpdateRelations.defaultValue,
        ...defaultValue,
      }

      // if(formNodeId) {
      //   submitData[formNodeId] = {
      //     ...(childUpdateRelations.submitData && childUpdateRelations.submitData[formNodeId]),
      //     ...submitData[formNodeId],
      //   }
      // }
      
      // submitData= {
      //   ...childUpdateRelations.submitData,
      //   ...submitData,
      // }

      //formNodeId = formNodeId || childUpdateRelations.formNodeId
    }
  })
  return {
    valueInSubmitData,
    submitData: resultData,
    //formNodeId,
    defaultValue,
  };
}

exports.buildEventHtml = ({
  eventContent = [],
  scopeNodeId = '',
  dataModels = {},
  relations = {},
  useDependencies = {},
  componentConfig = [],
  eventName,
}) => {
  let html = '';
  let updateState = {};
  eventContent.forEach(item => {
    if(['openModal', 'closeModal', 'state'].indexOf(item.eventType) >= 0 && (item.eventContent || item.stateNodeId))  {
      const contentValue = { closeModal: false, openModal: true };
      let defaultValue = {}

      if(item.eventContent) {
        updateState[`${item.eventContent}`] = contentValue[item.eventType];
      }

      if(item.stateNodeId && scopeNodeId) {
        updateState[item.stateNodeId] = `___${scopeNodeId}Scope___`;

        defaultValue = exports.pushDefaultValue({
          scopeNodeId,
          dataModels,
          relations,
          requiredRuleType: ['event'],
          getEventValue: (ruleValue) => {
            let value = '';
            if(ruleValue.length === 2) {
              value = `___${scopeNodeId}Scope.${ruleValue[1]}___`;
            }
            return value;
          }
        });
      }

      // 根据componentConfig，找到Modal元素，检查它的内部元素是否有Form, 最后更新Form内部有默认值的元素.
      if(item.eventType === 'openModal') {
        const submitData = {};
        const needUpdataData = exports.getFormValue({
          componentConfig,
          modalNodeId: item.modalNodeId,
          relations,
          dataModels,
          useDependencies,
          resultData: submitData,
          getApiValue: (ruleValue) => {
            let value = '';
            if(ruleValue.length === 2) {
              const stateName = exports.getApiParamStateName({dataModel: dataModels[ruleValue[0]]});
              value = `___this.state.${stateName} && this.state.${stateName}.${ruleValue[1]}___`;
            }
            return value;
          },
          getEventValue: item.stateNodeId && scopeNodeId ? (ruleValue) => {
            let value = '';
            if(ruleValue.length === 2) {
              value = `___${scopeNodeId}Scope.${ruleValue[1]}___`;
            }
            return value;
          } : (ruleValue) => {
            return '';
          }
        });

        Object.values(needUpdataData.valueInSubmitData).forEach(valueItem => {
          if(defaultValue[valueItem[0]] && defaultValue[valueItem[0]][valueItem[1]]) {
            delete defaultValue[valueItem[0]][valueItem[1]];
            if(!Object.keys(defaultValue[valueItem[0]]).length) {
              delete defaultValue[valueItem[0]];
            }
          }
        })

        Object.keys(submitData).forEach(formNodeId => {
          updateState[`${formNodeId}SubmitData`] = submitData[formNodeId];
        })
        

        defaultValue = {
          ...defaultValue,
          ...needUpdataData.defaultValue,
        }
      }
      
      // else if(item.eventType === 'closeModal') {
      //   defaultValue = {
      //     ...defaultValue,
      //     ...needUpdateValue,
      //   }
      //   console.log(33333333335, defaultValue);
      // }

      if(defaultValue && Object.keys(defaultValue).length) {
        html += `this.mergeState(${JSON.stringify(defaultValue).replace(/(\'|\")___/g, '').replace(/___(\'|\")/g, '')})\n`;
        useDependencies.mergeState = true;
      }
 
    }
    else if(['message'].indexOf(item.eventType) >= 0) {
      const time = `${item.messageDuration ? ','+item.messageDuration : ''}`;
      const msgHtml = `message.${item.eventContent}('${item.messageContent ? item.messageContent : '___res.msg___'}'${time});\n`;
      html += msgHtml.replace(/(\'|\")___/g, '').replace(/___(\'|\")/g, '');
    }
    else if(
      ['refresh'].indexOf(item.eventType) >= 0) {
      const relationService = item.relationService || [];
      relationService.forEach(serviceItem => {
        html += `this.${serviceItem.serviceName}(${JSON.stringify(serviceItem.params)});\n`
      })
    }
    else if(['page'].indexOf(item.eventType) >= 0 && item.eventContent) {
      const searchOption = [];
      
      if(item.pageParams && item.pageParams.length) {
        item.pageParams.forEach((pageParam) => {
          const paramScopeNodeId = exports.getNewNodeId(pageParam.scopeNodeId);
          searchOption.push(pageParam.value + '=${' + paramScopeNodeId + 'Scope.' + pageParam.value + '}')
        })
      }
      html += `this.props.history.push({
          pathname: '/${item.eventContent}',
          search: \`${item.eventContent.indexOf('?') >= 0 ? '&' : '?'}${searchOption.join('&')}\`,
        });\n`;
    }
    else if(['href'].indexOf(item.eventType) >= 0 && item.eventContent) {
      html += `window.open('${item.eventContent}');\n`
    }
    
  })
  if(Object.keys(updateState).length) {
    html += `this.setState(${JSON.stringify(updateState).replace(/(\'|\")___/g, '').replace(/___(\'|\")/g, '')})`;
  }

  if(eventName === 'submitSuccess') {
    html = `if(res.result === 'success') {
      ${html}
    }\n`;
  }
  else if(eventName === 'submitError') {
    html = `if(res.result === 'error') {
      ${html}
    }\n`;
  }

  return html;
}

exports.buildApiSuccessDefaultValue = ({
  dataModelsArray,
  dataModels,
  relations,
  useDependencies,
  componentConfig = []
}) => {
  dataModelsArray.forEach(dataModel => {
    let html = '';
    const defaultValue = exports.pushDefaultValue({
      serviceName: dataModel.dataKeyName,
      dataModels,
      relations,
      requiredRuleType: ['api'],
      getApiValue: (ruleValue) => {
        let value = '';
        if(ruleValue.length === 2) {
          value = `___resData.${ruleValue[1]}___`;
        }
        return value;
      }
    });

    const needUpdataData = {};
    exports.getFormValue({
      componentConfig,
      isInModal: true,
      relations,
      dataModels,
      useDependencies,
      resultData: needUpdataData,
      getApiValue: (ruleValue) => {
        let value = '';
        if(ruleValue.length === 2) {
          value = `___resData.${ruleValue[1]}___`;
        }
        return value;
      },
    });

    Object.keys(needUpdataData).forEach(formNodeId => {
      defaultValue[`${formNodeId}SubmitData`] = needUpdataData[formNodeId];
    })
    
  
    if(defaultValue && Object.keys(defaultValue).length) {
      html += `if(res.data && res.data.length) {
        const resData = res.data[0];
        this.mergeState(${JSON.stringify(defaultValue).replace(/(\'|\")___/g, '').replace(/___(\'|\")/g, '')})
      }\n`;
      useDependencies.mergeState = true;
    }
    dataModel.successDefaultValue = html;
  })
  
  return dataModelsArray;
}

exports.buildStaticDefaultValue = ({
  dataModels,
  relations,
}) => {
  let html = '';
  const defaultValue = exports.pushDefaultValue({
    dataModels,
    relations,
    requiredRuleType: ['static', 'pageParam'],
  });

  if(defaultValue && Object.keys(defaultValue).length) {
    html += `${JSON.stringify(defaultValue).replace(/(\'|\")___/g, '').replace(/___(\'|\")/g, '')}\n`;
  }
  return html;
}
/**
 * 在submit完成后，生成它绑定的事件refresh、message
 */
exports.buildEventHtmlForSubmit = ({
  dataModelEvents,
  dataModels,
  relations,
  useDependencies,
  componentConfig,
}) => {
  let key,subKey, htmls = {};
 
  for(key in dataModelEvents) {
    let html = '';
    for (subKey in dataModelEvents[key]) {
      if(['submitSuccess', 'submitError', 'submitComplete'].indexOf(subKey) >= 0) {
        html += this.buildEventHtml({
          eventName: subKey,
          eventContent: dataModelEvents[key][subKey],
          scopeNodeId: null,
          dataModels,
          relations,
          useDependencies,
          componentConfig,
        });
        delete dataModelEvents[key][subKey];
      }
    }
    htmls[key] = html;
  }
  return {
    htmls,
    dataModelEvents: dataModelEvents,
  };
}

exports.getDefaultValueParams = ({
  relations,
  dataModels,
}) => {
  const defaultValue = exports.pushDefaultValue({
    dataModels,
    relations,
    requiredRuleType: ['static', 'pageParam'],
  });

  return defaultValue;
}

exports.init = () => {
  nodeIds = {};
  newNodeIdCount = {};
  oldNodeIds = {};
  dataModelsNames = {};
}


